# check_setuid.pl
#
# Tom Edelson, 2003-09-16.
#
# Given SAS root on command line argument, checks whether the following
# files, in !SASROOT/utilities/bin, have root as owner and setuid permission. 
# If not, prompts for root password, and attempts to change them.

use strict;
use warnings;

use File::Spec;

use Exporter;

my ($success, $failure) = (0, 1);

my ($true, $false) = (1, 0);


my ($sasroot) = @ARGV;

my $ut_bin = File::Spec->catdir ($sasroot, 'utilities', 'bin');

my @files_to_check = ('elssrv', 'sasauth', 'sasperm');

my @need_changing = ();

my_printf ("\n");

foreach my $file (@files_to_check) {
    unless (already_set ($file)) {
        push @need_changing, $file;
    }
}

unless (scalar (@need_changing)) {
    my_printf ("All files already had correct permissions.\n");
    exit $success;
}

my $file_list = join ' ', @need_changing;

my $cmd = <<EOC;
su root -c "cd $ut_bin;
 chown root $file_list; 
 chmod 4755 $file_list"
EOC

### my_printf ($cmd);

my $YorN = <<EOM;
Unrecognized response. Please enter (y) to be prompted for root password
to complete this step or (n) to skip this step and perform it at a later
time.
EOM

my $continue = "Do you want to continue? (y) ";

my $try_again = "Do you want to try again? (y) ";

my $prompt1 = <<EOM;
You will be prompted for the root password, which will be used 
to set root ownership and setuid permission on these files 
in $ut_bin:

  $file_list

Note that without setting permissions on these files,
server configuration cannot proceed.

EOM

my $prompt2 = <<EOM;

Incorrect root password was entered.
EOM

my $utilities_auth_menu = <<EOM;

This hot fix will be installed without completing the server
configuration step. Server configuration can be performed at
a later date by following these steps:

Run sassetup

 From Primay Menu, select
 (2) Run Setup Utilities

 From Setup Utilities Menu, select
 (2) Perform SAS Software Configuration

 then  select "Configure User Authentication".

You will then be prompted to enter the root password to 
complete the configuration.
 
EOM

my_printf ($prompt1);
my_printf ($continue);

while ($true) {

    my $default = "y";
    my $reply = <STDIN>;
    $reply = $default if ($reply eq "\n");
    
    if ($reply =~ /^(y|Y)$/) {
       my_printf ("Enter root password:");
       my $rc = system ($cmd);
       $rc >>= 8;
       ### my_printf ("Return code was '$rc'.\n\n");
       if ($rc eq $success) {
           my_printf ("Permissions successfuly reset.\n");
           exit $success;
       }
       else {
          my_printf ($prompt2);
          my_printf ($try_again);
       }
    }
    elsif ($reply =~ /^(n|N)$/) {
       my_printf ($utilities_auth_menu);
       exit $success;
    }
    else{
       my_printf($YorN);
    }


}  # end while, and end main program.  


sub my_printf {
   my $message = shift;
   open FOUT, ">/dev/tty";
   $| = 1;
   printf FOUT "$message", @_ ;
   $| = 0;
   close FOUT;
   return;
}

sub already_set {

    my ($file) = @_;
    my $path = File::Spec->catfile ($ut_bin, $file);

    unless (-e $path) {
        my_printf ("Error: file does not exist: $path\n\n");
        exit $failure;
    }

    unless (-u $path){
        return $false;
    }

    my $owner = (stat $path)[4];
    $owner = getpwuid ($owner);
    if ($owner eq 'root') {
        return $true;
    }
    else {
        return $false;
    }

}


# end of file check_setuid.pl
