# setup_sas_scripts.pl
#
# Charles Nemargut, 2003-10-27.
#
# Given SAS root on command line argument. This program copies files from
# !SASROOT/bin/ to the !SASROOT/hotfix/bin directory and modifies them with
# the appropriate hotfix path where neccessary.
# 
# Updated: 07212005
#
#

use strict;
use warnings;

use File::Spec;
use File::Copy;
use File::Basename;

my ($success, $failure) = (0, 1);

my ($sasroot) = @ARGV;

my $sas_bin_dir = File::Spec->catdir ($sasroot, 'bin');
my $sas_in_sasroot = File::Spec->catfile ($sasroot, 'sas');
my $hotfix_bin_dir = File::Spec->catdir ($sasroot, 'hotfix', 'bin');
my $hotfix_script_dir = File::Spec->catdir($sasroot,'install','admin','hotfix',
                           'script');
my @hotfix_sasenv_files = qw(sasenv sasenv.ksh sasenv.csh);
my @script_names = ();


# Build a list of the language invocation script names.
opendir(SAS_BIN_DIR,$sas_bin_dir) or do{
   my_printf("\n Could not read from directory:$sas_bin_dir \n",$!);
   exit $success;
};
#my_printf("\n Opened directory:$sas_bin_dir \n");
# Let's create a hotfix bin if one does not already exist;
if (! -d $hotfix_bin_dir ) {
   mkdir $hotfix_bin_dir,0755 or do {
      my_printf("\n Could not create directory:$hotfix_bin_dir \n",$!);
   };
}
my $bin_script_file = "";
my $bin_file = "";
while ($bin_file = readdir(SAS_BIN_DIR)) {
   next if ($bin_file eq "."); # Skip current dir.
   next if ($bin_file eq ".."); # Skip parrent dir.
   next if ($bin_file =~ /sasenv/); # Skip All sasenv files.
   $bin_script_file = File::Spec->catfile($sas_bin_dir,$bin_file);
   next if (! -f $bin_script_file ); # Only want files not subdirs.
   push (@script_names,$bin_file);
}

closedir(SAS_BIN_DIR);


# Let's read the invocation scripts from !SASROOT/bin then make the appropriate
# path changes for hotfixes and write the modified files to !SASROOT/hotfix/bin

my $hf_script_file = "";
my $my_out_line = "";
foreach my $bin_script_name (@script_names) {
   $bin_script_file = File::Spec->catfile($sas_bin_dir,$bin_script_name);
   $hf_script_file = File::Spec->catfile($hotfix_bin_dir,$bin_script_name);
   open MYINFILE,"<$bin_script_file" or do {
      my_printf("\n Could not open file:$bin_script_file \n",$!)
   };
   my @all_the_lines = <MYINFILE>;
   close MYINFILE;
   open MYOUTFILE,">$hf_script_file" or do {
      my_printf("\n Could not create hotfix invocation file:$hf_script_file \n",$!);
      exit $failure;
   };
   #my_printf("\n Opened output file:$hf_script_file \n",$!);
   foreach my $the_line (@all_the_lines) {
      $my_out_line = $the_line;
      if ($my_out_line =~ /\$SAS_INSTALL_ROOT\/bin\/sasenv/ ){
         $my_out_line =~ s/\$SAS_INSTALL_ROOT\/bin\/sasenv/\$SAS_INSTALL_ROOT\/hotfix\/bin\/sasenv/g;
      }
      if ($my_out_line =~ /\$SAS_INSTALL_ROOT\/sasexe\/sas/ ) {
         if (-f "$sasroot/hotfix/sasexe/sas") {
            $my_out_line =~ s/\$SAS_INSTALL_ROOT\/sasexe\/sas/\$SAS_INSTALL_ROOT\/hotfix\/sasexe\/sas/g;
         }
      }
      print MYOUTFILE "$my_out_line";
   }
   close MYOUTFILE;
   chmod 0755, $hf_script_file or do {
      my_printf("\n Could not set permissions on file:$hf_script_file \n",$!);
      exit $failure;
   };
}

#my_printf("\n Hotfix language invocation scripts successfully created \n");

# We now need to create a symlink to the proper hotfix/bin invocation script.
my $sas_symlink = basename(readlink($sas_in_sasroot));
my $hf_bin_file = File::Spec->catfile($hotfix_bin_dir,$sas_symlink);
my $rel_path = "";
if (-f $hf_bin_file) {
   #remove existing symbolic link and create a new one.
   unlink $sas_in_sasroot or do {
      my_printf("\n Could not unlink:$sas_in_sasroot \n",$!);
   };
   my $new_path = dirname($sas_in_sasroot);
   $rel_path = File::Spec->abs2rel($hf_bin_file,$new_path);
   #symlink $hf_bin_file,$sas_in_sasroot or do{
   symlink $rel_path,$sas_in_sasroot or do{
      my_printf("\n Could not create symbolic link from $sas_in_sasroot to $hf_bin_file\n",$!);
   };
}

foreach my $hotfix_sasenv (@hotfix_sasenv_files) {
   my $check_hfbin_sasenv = File::Spec->catfile($hotfix_bin_dir,$hotfix_sasenv);
   my $hfscript_sasenv = File::Spec->catfile($hotfix_script_dir,$hotfix_sasenv);
   if (! -f $check_hfbin_sasenv) {
      # It doesn't exist so let's copy the file from the hotfix script area
      # if it exists there.
      if (-f $hfscript_sasenv) {
         # Let's copy the sasenv file over from the hotfix script area.
         copy($hfscript_sasenv,$check_hfbin_sasenv);
         chmod 0755, $check_hfbin_sasenv;
      }
   }
}
   
exit $success;

# END of Main routine.

sub my_printf {
   my $message = shift;
   open FOUT, ">/dev/tty";
   $| = 1;
   printf FOUT "$message", @_ ;
   $| = 0;
   close FOUT;
   return;
}


# end of file setup_sas_scripts.pl
