########################################################################
#
# xcflupdt.pl - Utility to update the Exclude File list for the 
#               SAS IQ process.
# 
# Notes:  This script is designed to be bundled and shipped as part of 
#         a hot fix.  It should be installed, used, and removed as part
#         of the hot fix installation process.
#
# Update Data File Syntax:
#
#  Each line that starts with a '+' is a line to be added (add-line).
#  Each line that does not start with a '+' is a "line-up" identifier.
#
#  The line-up identifier lines are used to locate a relative location
# in the existing exclude file.  The more adjacent line-up lines, the
# higher the degree of accuracy of placing the next set of lines to be
# added in the correct place.
#
#  The file will be parsed as follows.  Each line in the udpate file is
# parsed one at a time.  For each line, the target exclude file is searched
# (initially from the top, then starting from the last matching line-up
# line) looking for a matching line-up line.  When an add-line is found,
# it is inserted in the exclude file at the 'current location'.  If an 
# add-line is the first line in the update file, then the add-line will
# be inserted as the first line in the exclude file.  Otherwise, it will
# be inserted after the last matching line-up line.
#
#  If a line-up line does not match any line in the (remaining content) of
# the target exclude file (i.e. EOF is found for the exlude file when 
# searching for a line-up match), then this is considered an error condition.
# The new exclude file will NOT be written and an error will be produced.
#
#  Note that adjacent line-up lines in the update file do not have to be 
# adjacent in the target exclude file.  Each line search is independent
# of the previous.  The target exclude file is only parsed from top to 
# bottom once.  So line-up lines in the update file do have to be in 
# the same relative order as expected in the target exclude file.
#
#  Add-lines are denoted by '+' signs.  When added to the exclude file, 
# the '+' sign is removed.
#
#  This design has been extended to include subtract-lines as well.  Those
# lines are denoted by a '-' sign as the first character.  Note that no
# other characters on the subtract-line matters.  If a subtract-line is
# seen, the last seen line of the exclude file will be removed.  You can
# have multiple subtract-lines in a row to remove multiple lines in the 
# exclude file.  
#
########################################################################

package xcflupdt;

use strict;

use vars qw(@EXPORT @ISA);

use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(UpdateFile);

sub UpdateFile 
{
  my $DataFilePath = shift;
  my $exclFile = shift;

  if (! defined $DataFilePath || ! defined $exclFile) {
    print "Usage:  xcflupdt <UpdateDataFile> <ExcludeFile>\n";
    print "       <UpdateDataFile> is the pathname of a file that holds the update rules.\n";
    print "       <ExcludeFile> is the pathname of the target exclude file to update.\n";
    print "\n";
    print "Example:  xcflupdt install/admin/hotfix/script/b9ba03.xcflupdat.txt sastest/sasiq_excludeFiles.xml\n";
    return 1;
  }

  #=============================================================================
  # 1) Locate the target exclude file, open it for reading, 
  #    slurp the whole file in, and close it.
  if (! -e $exclFile) {
    print STDERR "xcflupdt:  Error - Unable to locate exclude file:  $exclFile.\n";
    return 10;
  }

  if (!open(EXCLUDE, "$exclFile")) {
    print STDERR "xcflupdt:  Error - Unable to open exclude file for reading:  $exclFile.\n";
    return 11;
  }

  my @ExclFileLines = <EXCLUDE>;
  close EXCLUDE;

  #=============================================================================
  # 2) Locate the target update date file, open it for reading, 
  #    slurp the whole file in, and close it.
  if (! -e $DataFilePath) {
    print STDERR "xcflupdt:  Error - Unable to locate update data file:  $DataFilePath.\n";
    return 12;
  }

  if (!open(UPDATE, "$DataFilePath")) {
    print STDERR "xcflupdt:  Error - Unable to open update data file for reading:  $DataFilePath.\n";
    return 13;
  }
  my @updateLines = <UPDATE>;
  close UPDATE;

  #=============================================================================
  # 3) Parse the exclude file and the target file
  #    simultaneously.  Build a new file in memory
  #    that represents the new exclude file.
  my @newExcludeFileLines = ();

  my $rc = 0;
  my $nextTargetLine;
  foreach my $nextLine (@updateLines)
  {
    chomp $nextLine;

    # Skip blank lines.
    next if ($nextLine =~ /^\s*$/);

    if ($nextLine =~ /^\+(.*)/) 
    {
      # This is an add-line.
      push(@newExcludeFileLines, "$1\n");
    } 
    elsif ($nextLine =~ /^\-/) 
    {
      # This is a subtract-line.
      pop(@newExcludeFileLines);
    }
    else
    {
      while (1) 
      {
        # This is a line-up line.
        $nextTargetLine = shift @ExclFileLines;
        chomp($nextTargetLine);

        if (! defined $nextTargetLine)
        { 
          # EOF - This should never be found.
          print STDERR "xcflupdt:  Error - Unable to locate matching exclude file line for:  $nextLine.\n";
          $rc = 11;
          last;
        }

        # Add this line to the new exclude file.
        push(@newExcludeFileLines, "$nextTargetLine\n");

        # See if this is a match for the next update line-up line.
        last if ($nextTargetLine eq $nextLine);
      }
    }

    last if ($rc != 0);
  }

  # If there are more @ExclFileLines, cat them on @newExcludeFileLines.
  push @newExcludeFileLines, @ExclFileLines;

  #=============================================================================
  # 4)  Open the target exclude file for writing, write out the new
  #     contents, and close it.
  if ($rc == 0)
  {
    if (!open(EXCLUDE, ">$exclFile")) {
      print STDERR "xcflupdt:  Error - Unable to open exclude file for writing:  $exclFile.\n";
      return 14;
    }

    # Write the new exclude file.
    print EXCLUDE @newExcludeFileLines;

    # Close all open files.
    close EXCLUDE;
  } else {
    print STDERR "xcflupdt:  Error - Unable to apply the Update Data with success.  Exclude file unchanged.\n";
    return 15;
  }

  #=============================================================================
  return $rc;
}

1;  # end of file startup.pm
